Imports Telerik.WinControls
Imports Telerik.WinControls.Primitives

Public Class TextRotatorElement
    Inherits RadItem

#Region "Private fields"

    Private textPrimitive As TextPrimitive

    Private _strings As String()
    Private _currentString As Integer = -1

    Private _timer As Timer

    Private _scale As Double = 1
    Private _angle As Double = 0
    Private _steps As Integer = 37
    Private _step As Double

#End Region

#Region "Constructors & Initialization"

    Public Sub New()
        Me.AutoSize = True
        Me.StretchHorizontally = False
        Me.StretchVertically = False
        Me.Alignment = ContentAlignment.MiddleCenter
        _timer = New Timer()
        _timer.Interval = 20
        AddHandler _timer.Tick, AddressOf timer_Tick

        Me._step = Math.PI / Me._steps
    End Sub

    Protected Overrides Sub CreateChildElements()
        textPrimitive = New TextPrimitive()
        Dim fill As FillPrimitive = New FillPrimitive()

        Me.Children.Add(fill)
        Me.Children.Add(textPrimitive)
    End Sub

#End Region

#Region "Properties"

    Public Property Strings() As String()
        Get
            If (Not Me._strings Is Nothing) Then
                Return Me._strings
            Else
                Return New String() {String.Empty}
            End If
        End Get
        Set(ByVal value As String())
            Me._strings = value

            If Me._strings.Length > 0 Then
                _currentString = 0
                Me.textPrimitive.Text = Me._strings(0)
            Else
                _currentString = -1
                Me.textPrimitive.Text = String.Empty
            End If
        End Set
    End Property

    Public ReadOnly Property Running() As Boolean
        Get
            Return Me._timer.Enabled
        End Get
    End Property

    Public Property Steps() As Integer
        Get
            Return Me._steps
        End Get
        Set(ByVal value As Integer)
            If Me._steps = value Then
                Return
            End If

            Me._steps = value
            Me._step = Math.PI / Me._steps
        End Set
    End Property

#End Region

#Region "Public Methods"

    Public Sub Start()
        _timer.Start()
    End Sub

    Public Sub [Stop]()
        _timer.Stop()
    End Sub

#End Region

    Private Sub timer_Tick(ByVal sender As Object, ByVal e As EventArgs)
        If Me._strings.Length = 0 Then
            _timer.Stop()
            Return
        End If

        _angle += _step
        Dim newScale As Single = CSng(Math.Cos(_angle))

        If Math.Sign(Me._scale) <> Math.Sign(newScale) Then
            ' Advance to the next string
            _currentString = (_currentString + 1) Mod _strings.Length
            textPrimitive.Text = _strings(_currentString)
        End If

        ' Take care for the case when the Scale is too small (less than 1 pixel)
        ' Otherwise GDI+ exception is thrown
        If Math.Abs(newScale * Me.Size.Width) < 1 Then
            Me._scale = Math.Sign(newScale) / Me.Size.Width
        Else
            Me._scale = newScale
        End If

        Me.ScaleTransform = New SizeF(CSng(Math.Abs(Me._scale)), 1)

    End Sub

    Protected Overrides Function MeasureOverride(ByVal availableSize As SizeF) As SizeF
        Return MyBase.MeasureOverride(New SizeF(Single.PositiveInfinity, Single.PositiveInfinity))
    End Function

End Class
